/*
* Copyright 2009-2019  NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO USER:
*
* This source code is subject to NVIDIA ownership rights under U.S. and
* international Copyright laws.
*
* This software and the information contained herein is PROPRIETARY and
* CONFIDENTIAL to NVIDIA and is being provided under the terms and conditions
* of a form of NVIDIA software license agreement.
*
* NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
* CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
* IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
* REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
* MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
* IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
* OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
* OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
* OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
* OR PERFORMANCE OF THIS SOURCE CODE.
*
* U.S. Government End Users.   This source code is a "commercial item" as
* that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
* "commercial computer  software"  and "commercial computer software
* documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
* and is provided to the U.S. Government only as a commercial end item.
* Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
* 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
* source code with only those rights set forth herein.
*
* Any use of this source code in individual and commercial software must
* include, in the user documentation and internal comments to the code,
* the above Disclaimer and U.S. Government End Users Notice.
*/

#ifndef NVTOOLSEXT_EXTENSION_JSON_V3
#define NVTOOLSEXT_EXTENSION_JSON_V3

/* Use the same pattern as nvToolsExt.h for including stdint.h and stddef.h */
#ifndef NVTX_STDINT_TYPES_ALREADY_DEFINED
#include <stdint.h>
#endif
#include "stddef.h"  /* For uintptr_t definition */

/*  ------------------------------------------------------------------------- */
/** \brief Additional values for the enum nvtxPayloadType_t
*
* These values are intended to be extensions to the enum
* nvtxPayloadType_t, which is used for "payloadType" in
* nvtxEventAttributes_t.  Assign to "payload.ullValue"
* using the helper macro NVTX_POINTER_AS_PAYLOAD_ULLVALUE.
*
* For example:
*     const char* jsonPayload = "[\"Hello\", \"World\"]";
*     nvtxEventAttributes_t eventAttr = {0};
*     eventAttr.payloadType = NVTX_PAYLOAD_JSON;
*     eventAttr.payload.ullValue = NVTX_POINTER_AS_PAYLOAD_ULLVALUE(jsonPayload);
*/
typedef enum nvtxPayloadTypeExtensionJson_t
{
    /* Additional values for the enum nvtxMessageType_t */
    NVTX_PAYLOAD_TYPE_JSON             = 7,     /**< Payload is a pointer to a null-terminated
                                                 ASCII or UTF-8 string containing JSON data, as
                                                 specified in RFC 8259.  Tools shall define how
                                                 they display or use this JSON data.  No schema
                                                 restrictions are imposed by the NVTX API.  When
                                                 assigning a string to an nvtxEventAttributes_t
                                                 "payload" union, use the "ullValue" member
                                                 and cast the pointer using the helper macro
                                                 NVTX_POINTER_AS_PAYLOAD_ULLVALUE. */
    NVTX_PAYLOAD_TYPE_JSON_REGISTERED  = 8      /**< A string handle obtained from calls such as
                                                 \ref nvtxDomainRegisterStringA().  The data in
                                                 the registered string will be interpreted by
                                                 tools as JSON, as specified in RFC 8259.  When
                                                 assigning to an nvtxEventAttributes_t "payload"
                                                 union, use the "ullValue" member and cast the
                                                 registered string handle using the helper macro
                                                 NVTX_STRING_HANDLE_AS_PAYLOAD_ULLVALUE. */
} nvtxPayloadTypeExtensionJson_t;

/* Helper macros for safe double-cast of pointer to uint64_t value */
#ifdef __cplusplus
#define NVTX_POINTER_AS_PAYLOAD_ULLVALUE(p) static_cast<uint64_t>(reinterpret_cast<uintptr_t>(p))
#else
#define NVTX_POINTER_AS_PAYLOAD_ULLVALUE(p) ((uint64_t)(uintptr_t)p)
#endif
#define NVTX_STRING_HANDLE_AS_PAYLOAD_ULLVALUE(h) NVTX_POINTER_AS_PAYLOAD_ULLVALUE(h)

#endif /* NVTOOLSEXT_EXTENSION_JSON_V3 */
